//
// The MIT License (MIT)
//
// Copyright (c) 2017 Tommaso Madonia
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.
//

// MARK: - Logger protocol

public protocol Logger {

    var enabled: Bool { get set }
    var level: LogLevel { get set }

    func debug(_ message: @autoclosure () -> String)
    func info(_ message: @autoclosure () -> String)
    func warn(_ message: @autoclosure () -> String)
    func error(_ message: @autoclosure () -> String)
    
}

// MARK: - LoggerLevel enum

public enum LogLevel: CustomStringConvertible, Comparable {

    case debug, info, warning, error

    public var description: String {
        switch self {
            case .debug: return "DEBUG"
            case .info: return "INFO"
            case .warning: return "WARN"
            case .error: return "ERROR"
        }
    }
    
    public static func <(lhs: LogLevel, rhs: LogLevel) -> Bool {
        switch (lhs, rhs) {
        case (let lhs, .debug) where lhs != .debug: return true
        case (.warning, .info), (.error, .info): return true
        case (.error, .warning): return true
        default: return false
        }
    }

}
